function [X_all, f_all, s_all] = P2GDRinfo(f, g_f, X, r, a0, b, c, Delta, epsilon, i_max)
%% Description
% This function implements [OGA22, Algorithm 2] (https://arxiv.org/abs/2201.03962v1).
% It stops as soon as the stationarity measure becomes smaller than or equal to epsilon.
% Input arguments:
%   - f, the cost function;
%   - g_f, the gradient of the cost function;
%   - X, the initial iterate;
%   - r, the rank of the considered determinantal variety;
%   - a0, the initial step size for the backtracking line search;
%   - b, the update factor for the backtracking line search;
%   - c, the Armijo parameter;
%   - Delta, the rank reduction parameter;
%   - epsilon, the tolerance on the stationarity measure;
%   - i_max, the maximum number of iterations.
% Output arguments:
%   - X_all, a cell containing all the iterates produced by the algorithm;
%   - f_all, a vector containing the values of the cost function at all the iterations;
%   - s_all, a vector containing the values of the stationarity measure at all the iterations.
%% Code
% Convergence tolerance for svds
OPTIONS.tol = 1e-15;
% Initialization
i = 1;
f_X = f(X);
f_all = zeros(i_max,1);
s_all = zeros(i_max,1);
X_all = cell([i_max 1]);
f_all(1) = f_X;
X_all{1} = X;
% Compact SVD of X
[U, s, V] = svds(X, r, 'largest', OPTIONS);
s = diag(s);
s = s(s > 0);
r_X = length(s);
U = U(:,1:r_X);
V = V(:,1:r_X);
% Orthogonal projectors onto im X and im X'
PU = U*U';
PV = V*V';
% Negative gradient
G0 = -g_f(X);
% Projection of the negative gradient onto the tangent space at X
G1 = PU*G0 + G0*PV - PU*G0*PV;
if r_X < r % the tangent cone is not the tangent space
    % Low-rank approximation of the normal part of the negative gradient
    [U2, S2, V2] = svds(G0-G1, r-r_X, 'largest', OPTIONS);
    G2 = U2*S2*V2';
    G = G1+G2;
else
    G = G1;
end
% Stationarity measure at X
s_X = norm(G,'fro');
s_all(1) = s_X;
s2_X = s_X*s_X;
while s_X > epsilon && i < i_max
    % Backtracking line search
    a = a0;
    [U_Y, S_Y, V_Y] = svds(X+a*G, r, 'largest', OPTIONS);
    Y = U_Y*S_Y*V_Y';
    f_Y = f(Y);
    while f_Y > f_X - c*a*s2_X
        a = a*b;
        [U_Y, S_Y, V_Y] = svds(X+a*G, r, 'largest', OPTIONS);
        Y = U_Y*S_Y*V_Y';
        f_Y = f(Y);
    end
    Y_ref = Y;
    f_ref = f_Y;
    % Delta-rank of X
    s_Delta = s(s > Delta);
    r_Delta = length(s_Delta);
    r_diff = r_X-r_Delta;
    % Rank reduction
    if r_diff > 0
        fprintf('rank reduction considered at iteration %d\n',i-1)
        for j = 1:r_diff
            r_j = r_X-j+1; % = rank X^j + 1
            X = X-U(:,r_j)*s(r_j)*V(:,r_j)'; % = X^j
            PU = PU-U(:,r_j)*U(:,r_j)';
            PV = PV-V(:,r_j)*V(:,r_j)';
            G0 = -g_f(X);
            G1 = PU*G0 + G0*PV - PU*G0*PV;
            [U2, S2, V2] = svds(G0-G1, r-r_j+1, 'largest', OPTIONS);
            G2 = U2*S2*V2';
            G = G1+G2;
            s_X = norm(G,'fro'); % stationarity measure at X
            s2_X = s_X*s_X;
            % Backtracking line search
            a = a0;
            [U_Y, S_Y, V_Y] = svds(X+a*G, r, 'largest', OPTIONS);
            Y = U_Y*S_Y*V_Y';
            f_X = f(X);
            f_Y = f(Y);
            while f_Y > f_X - c*a*s2_X
                a = a*b;
                [U_Y, S_Y, V_Y] = svds(X+a*G, r, 'largest', OPTIONS);
                Y = U_Y*S_Y*V_Y';
                f_Y = f(Y);
            end
            if f_Y < f_ref
                fprintf('reduction to rank %d used at iteration %d\n',r_j-1,i-1)
                Y_ref = Y;
                f_ref = f_Y;
            end
        end
    end
    X = Y_ref;
    f_X = f_ref;
    % Compact SVD of X
    [U, s, V] = svds(X, r, 'largest', OPTIONS);
    s = diag(s);
    s = s(s > 0);
    r_X = length(s);
    U = U(:,1:r_X);
    V = V(:,1:r_X);
    % Orthogonal projectors onto im X and im X'
    PU = U*U';
    PV = V*V';
    % Negative gradient
    G0 = -g_f(X);
    % Projection of the negative gradient onto the tangent space at X
    G1 = PU*G0 + G0*PV - PU*G0*PV;
    if r_X < r % the tangent cone is not the tangent space
        % Low-rank approximation of the normal part of the negative gradient
        [U2, S2, V2] = svds(G0-G1, r-r_X, 'largest', OPTIONS);
        G2 = U2*S2*V2';
        G = G1+G2;
    else
        G = G1;
    end
    % Stationarity measure at X
    s_X = norm(G,'fro');
    s2_X = s_X*s_X;
    i = i+1;
    X_all{i} = X;
    f_all(i) = f_X;
    s_all(i) = s_X;
end
f_all = f_all(1:i);
s_all = s_all(1:i);
X_all = X_all(1:i);
end